@@ -14,10 +14,13 @@
 #ifndef TIG_IO_H
 #define TIG_IO_H
 
+#include "tig.h"
+
 /*
  * Argument array helpers.
  */
 
+bool argv_to_string(const char *argv[SIZEOF_ARG], char *buf, size_t buflen, const char *sep);
 bool argv_from_string_no_quotes(const char *argv[SIZEOF_ARG], int *argc, char *cmd);
 bool argv_from_string(const char *argv[SIZEOF_ARG], int *argc, char *cmd);
 bool argv_from_env(const char **argv, const char *name);
@@ -26,6 +29,7 @@ size_t argv_size(const char **argv);
 bool argv_append(const char ***argv, const char *arg);
 bool argv_append_array(const char ***dst_argv, const char *src_argv[]);
 bool argv_copy(const char ***dst, const char *src[]);
+bool argv_remove_quotes(const char *argv[]);
 
 /*
  * Encoding conversion.
@@ -45,6 +49,7 @@ enum io_type {
 	IO_BG,			/* Execute command in the background. */
 	IO_FG,			/* Execute command with same std{in,out,err}. */
 	IO_RD,			/* Read only fork+exec IO. */
+	IO_RD_STDIN,		/* Read only fork+exec IO with stdin. */
 	IO_WR,			/* Write only fork+exec IO. */
 	IO_AP,			/* Append fork+exec output to file. */
 };
@@ -63,7 +68,7 @@ struct io {
 
 typedef int (*io_read_fn)(char *, size_t, char *, size_t, void *data);
 
-bool io_open(struct io *io, const char *fmt, ...);
+bool io_open(struct io *io, const char *fmt, ...) PRINTF_LIKE(2, 3);
 bool io_kill(struct io *io);
 bool io_done(struct io *io);
 bool io_run(struct io *io, enum io_type type, const char *dir, const char *argv[], ...);
@@ -77,7 +82,7 @@ bool io_can_read(struct io *io, bool can_block);
 ssize_t io_read(struct io *io, void *buf, size_t bufsize);
 char * io_get(struct io *io, int c, bool can_read);
 bool io_write(struct io *io, const void *buf, size_t bufsize);
-bool io_printf(struct io *io, const char *fmt, ...);
+bool io_printf(struct io *io, const char *fmt, ...) PRINTF_LIKE(2, 3);
 bool io_read_buf(struct io *io, char buf[], size_t bufsize);
 bool io_run_buf(const char **argv, char buf[], size_t bufsize);
 int io_load(struct io *io, const char *separators,
